@@ -35,18 +35,32 @@ using Beagle.Util;
 
 namespace Beagle {
 
-	public enum IndexableFiltering {
-		Never,     // Never try to filter this indexable
-		Automatic, // Try to determine automatically if this needs to be filtered
-		Always     // Always try to filter this indexable
+	public enum IndexableType {
+		Add,
+		Remove,
+		PropertyChange
 	}
 
-	public class Indexable : Versioned {
+	public enum IndexableFiltering {
+		Never,           // Never try to filter this indexable, it contains no content
+		AlreadyFiltered, // The readers promise to return nice clean text, so do nothing
+		Automatic,       // Try to determine automatically if this needs to be filtered
+		Always           // Always try to filter this indexable
+	}
 
+	public class Indexable : Versioned, IComparable {
+
+		// This is the type of indexing operation represented by
+		// this Indexable object.  We default to Add, for historical
+		// reasons.
+		private IndexableType type = IndexableType.Add;
 
 		// The URI of the item being indexed.
 		private Uri uri = null;
 
+		// The URI of the parent indexable, if any.
+		private Uri parent_uri = null;
+
 		// The URI of the contents to index
 		private Uri contentUri = null;
 
@@ -57,7 +71,7 @@ namespace Beagle {
 		private bool deleteContent = false;
 		
 		// File, WebLink, MailMessage, IMLog, etc.
-		private String type = null;
+		private String hit_type = null;
 
 		// If applicable, otherwise set to null.
 		private String mimeType = null;
@@ -82,9 +96,16 @@ namespace Beagle {
 		// A stream of the hot content to index
 		private TextReader hotTextReader;
 
+		// A stream of binary data to filter
+		private Stream binary_stream;
+
 		// When should we try to filter this indexable?
 		private IndexableFiltering filtering = IndexableFiltering.Automatic;
 
+		// Local state: these are key/value pairs that never get serialized
+		// into XML
+		Hashtable local_state = new Hashtable ();
+
 		//////////////////////////
 
 		static private XmlSerializer our_serializer;
@@ -96,13 +117,19 @@ namespace Beagle {
 
 		//////////////////////////
 
-		public Indexable (Uri _uri) {
-			uri = _uri;
-
-			type = "File";
+		public Indexable (IndexableType type,
+				  Uri           uri)
+		{
+			this.type = type;
+			this.uri = uri;
+			this.hit_type = "File"; // FIXME: Why do we default to this?
 		}
 
-		public Indexable () {
+		public Indexable (Uri uri) : this (IndexableType.Add, uri)
+		{ }
+
+		public Indexable ()
+		{
 			// Only used when reading from xml
 		}
 
@@ -114,6 +141,12 @@ namespace Beagle {
 
 		//////////////////////////
 
+		[XmlAttribute ("Type")]
+		public IndexableType Type {
+			get { return type; }
+			set { type = value; }
+		}
+
 		[XmlIgnore]
 		public Uri Uri { 
 			get { return uri; }
@@ -126,6 +159,29 @@ namespace Beagle {
 			set { uri = UriFu.UriStringToUri (value); }
 		}
 
+		[XmlIgnore]
+		public Uri ParentUri { 
+			get { return parent_uri; }
+			set { parent_uri = value; }
+		}
+
+		[XmlAttribute ("ParentUri")]
+		public string ParentUriString {
+			get {
+				if (parent_uri == null)
+					return null;
+
+				return UriFu.UriToSerializableString (parent_uri);
+			}
+
+			set {
+				if (value == null)
+					parent_uri = null;
+				else
+					parent_uri = UriFu.UriStringToUri (value);
+			}
+		}
+
 		[XmlIgnore]
 		public Uri ContentUri {
 			get { return contentUri != null ? contentUri : Uri; }
@@ -152,7 +208,7 @@ namespace Beagle {
 
 		[XmlIgnore]
 		public Uri DisplayUri {
-			get { return uri.Scheme == "uid" ? ContentUri : Uri; }
+			get { return uri.Scheme == GuidFu.UriScheme ? ContentUri : Uri; }
 		}
 
 		[XmlAttribute]
@@ -162,9 +218,9 @@ namespace Beagle {
 		}
 
 		[XmlAttribute]
-		public String Type {
-			get { return type; }
-			set { type = value; }
+		public String HitType {
+			get { return  hit_type; }
+			set { hit_type = value; }
 		}
 
 		[XmlAttribute]
@@ -175,7 +231,7 @@ namespace Beagle {
 
 		[XmlIgnore]
 		public bool IsNonTransient {
-			get { return ! DeleteContent && ContentUri.IsFile; }
+			get { return ! DeleteContent && ContentUri.IsFile && ParentUri == null; }
 		}
 
 		[XmlAttribute]
@@ -202,30 +258,69 @@ namespace Beagle {
 			set { filtering = value; }
 		}
 
+		[XmlIgnore]
+		public IDictionary LocalState {
+			get { return local_state; }
+		}
+
 		//////////////////////////
-		
-		private TextReader ReaderFromUri (Uri uri)
+
+		public void Cleanup ()
 		{
-			TextReader reader = null;
+			if (DeleteContent) {
+				if (contentUri != null) {
+					//Logger.Log.Debug ("Cleaning up {0}", contentUri.LocalPath);
+					try {
+						File.Delete (contentUri.LocalPath);
+					} catch (Exception ex)
+					{ 
+						// It might be gone already, so catch the exception.
+					}
+					contentUri = null;
+				}
+				if (hotContentUri != null) {
+					//Logger.Log.Debug ("Cleaning up {0}", hotContentUri.LocalPath);
+					try {
+						File.Delete (hotContentUri.LocalPath);
+					} catch (Exception ex)
+					{
+						// Ditto
+					}
+					hotContentUri = null;
+				}
+			}
+		}
+
+		private Stream StreamFromUri (Uri uri)
+		{
+			Stream stream = null;
 
 			if (uri != null && uri.IsFile && ! no_content) {
-				Stream stream = new FileStream (uri.LocalPath,
-								FileMode.Open,
-								FileAccess.Read,
-								FileShare.Read);
-
-				reader = new StreamReader (stream);
-
-				// Paranoia: never delete the thing we are actually indexing.
-				if (DeleteContent && uri != Uri)
-					File.Delete (uri.LocalPath);
+				stream = new FileStream (uri.LocalPath,
+							 FileMode.Open,
+							 FileAccess.Read,
+							 FileShare.Read);
 			}
 
-			return reader;
+			return stream;
 		}
 
+		private TextReader ReaderFromUri (Uri uri)
+		{
+			Stream stream = StreamFromUri (uri);
+
+			if (stream == null)
+				return null;
+
+			return new StreamReader (stream);
+		}
+				
+
 		public TextReader GetTextReader ()
 		{
+			if (NoContent)
+				return null;
+
 			if (textReader == null)
 				textReader = ReaderFromUri (ContentUri);
 
@@ -239,6 +334,9 @@ namespace Beagle {
 
 		public TextReader GetHotTextReader ()
 		{
+			if (NoContent)
+				return null;
+
 			if (hotTextReader == null)
 				hotTextReader = ReaderFromUri (HotContentUri);
 			return hotTextReader;
@@ -249,39 +347,91 @@ namespace Beagle {
 			hotTextReader = reader;
 		}
 
+		public Stream GetBinaryStream ()
+		{
+			if (NoContent)
+				return null;
+
+			if (binary_stream == null)
+				binary_stream = StreamFromUri (ContentUri);
+
+			return binary_stream;
+		}
+
+		public void SetBinaryStream (Stream stream)
+		{
+			binary_stream = stream;
+		}
+
 		[XmlArrayItem (ElementName="Property", Type=typeof (Property))]
 		public ArrayList Properties {
 			get { return properties; }
 		}
 
 		public void AddProperty (Property prop) {
-			properties.Add (prop);
+			if (prop != null) {
+				
+				if (type == IndexableType.PropertyChange && ! prop.IsMutable)
+					throw new ArgumentException ("Non-mutable properties aren't allowed in this indexable");
+
+				// If this is a mutable property, make sure that
+				// we don't already contain another mutable property
+				// with the same name.  If we do, replace it.
+				if (prop.IsMutable) {
+					for (int i = 0; i < properties.Count; ++i) {
+						Property other_prop = properties [i] as Property;
+						if (other_prop.IsMutable && prop.Key == other_prop.Key) {
+							properties [i] = prop;
+							return;
+						}
+					}
+				}
+
+				properties.Add (prop);
+			}
+		}
+
+		public bool HasProperty (string keyword) {
+			foreach (Property property in properties)
+				if (property.Key == keyword)
+					return true;
+
+			return false;
+		}
+
+		// This doesn't check if it makes sense to actually
+		// merge the two indexables: it just does it.
+		public void Merge (Indexable other)
+		{
+			this.Timestamp = other.Timestamp;
+
+			foreach (Property prop in other.Properties)
+				this.AddProperty (prop);
+
+			foreach (DictionaryEntry entry in other.local_state)
+				this.local_state [entry.Key] = entry.Value;
 		}
 
 		//////////////////////////
 
-		private string PropertiesAsString (bool keywords)
+		public void SetChildOf (Indexable parent)
 		{
-			StringBuilder sb = new StringBuilder ();
-			foreach (Property prop in Properties) {
-				if (prop.IsSearched 
-				    && keywords ? prop.IsKeyword : ! prop.IsKeyword) {
-					if (sb.Length > 0)
-						sb.Append (" ");
-					sb.Append (prop.Value);
-				}
+			this.ParentUri = parent.Uri;
+
+			if (!this.ValidTimestamp)
+				this.Timestamp = parent.Timestamp;
+
+			// FIXME: Set all of the parent's properties on the
+			// child so that we get matches against the child
+			// that otherwise would match only the parent, at
+			// least until we have proper RDF support.
+			foreach (Property prop in parent.Properties) {
+				Property new_prop = (Property) prop.Clone ();
+				new_prop.Key = "parent:" + new_prop.Key;
+				this.AddProperty (new_prop);
 			}
-			return sb.ToString ();
 		}
 
-		public string TextPropertiesAsString {
-			get { return PropertiesAsString (false); }
-		}
-
-		public string KeywordPropertiesAsString {
-			get { return PropertiesAsString (true); }
-		}
-		
 		//////////////////////////
 
 		public override string ToString () 
@@ -327,14 +477,54 @@ namespace Beagle {
 			return UriFu.PathToFileUri (filename);
 		}
 
+		private static Uri BinaryStreamToTempFileUri (Stream stream)
+		{
+			if (stream == null)
+				return null;
+
+			string filename = Path.GetTempFileName ();
+			FileStream fileStream = File.OpenWrite (filename);
+
+			// When we dump the contents of an indexable into a file, we
+			// expect to use it again soon.
+			FileAdvise.PreLoad (fileStream);
+
+			// Make sure the temporary file is only readable by the owner.
+			// FIXME: There is probably a race here.  Could some malicious program
+			// do something to the file between creation and the chmod?
+			Mono.Posix.Syscall.chmod (filename, (Mono.Posix.FileMode) 256);
+
+			BufferedStream bufferedStream = new BufferedStream (fileStream);
+
+			const int BUFFER_SIZE = 8192;
+			byte [] buffer = new byte [BUFFER_SIZE];
+
+			int read;
+			do {
+				read = stream.Read (buffer, 0, BUFFER_SIZE);
+				if (read > 0)
+					bufferedStream.Write (buffer, 0, read);
+			} while (read > 0);
+
+			bufferedStream.Close ();
+
+			return UriFu.PathToFileUri (filename);
+		}
+
 		public void StoreStream () {
 			if (textReader != null) {
 				ContentUri = TextReaderToTempFileUri (textReader);
+				//Logger.Log.Debug ("Storing text content from {0} in {1}", Uri, ContentUri);
+				DeleteContent = true;
+			} else if (binary_stream != null) {
+				ContentUri = BinaryStreamToTempFileUri (binary_stream);
+				//Logger.Log.Debug ("Storing binary content from {0} in {1}", Uri, ContentUri);
 				DeleteContent = true;
 			}
 
 			if (hotTextReader != null) {
 				HotContentUri = TextReaderToTempFileUri (hotTextReader);
+				//Logger.Log.Debug ("Storing hot content from {0} in {1}", Uri, HotContentUri);
 				DeleteContent = true;
 			}
 		}
@@ -345,5 +535,11 @@ namespace Beagle {
 		{
 			return (uri != null ? uri.GetHashCode () : 0) ^ type.GetHashCode ();
 		}
+
+		public int CompareTo (object obj)
+		{
+			Indexable other = (Indexable) obj;
+			return DateTime.Compare (this.Timestamp, other.Timestamp);
+		}
 	}
 }
