@@ -1,4 +1,3 @@
-
 //
 // EvolutionMailDriver.cs
 //
@@ -34,6 +33,8 @@ using System.Threading;
 using Beagle.Util;
 using Camel = Beagle.Util.Camel;
 
+using LNI = Lucene.Net.Index;
+
 namespace Beagle.Daemon.EvolutionMailDriver {
 
 	[QueryableFlavor (Name="Mail", Domain=QueryDomain.Local, RequireInotify=false)]
@@ -41,39 +42,33 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 
 		public int polling_interval_in_seconds = 60;
 
-		public static Logger log = Logger.Get ("mail");
 		private string local_path, imap_path, imap4_path;
 
-		private SortedList watched = new SortedList ();
 		private MailCrawler crawler;
 
-		private object lockObj = new object ();
-		private ArrayList AddedUris = new ArrayList ();
-		private bool queryRunning = false;
+		// Index versions
+		// 1: Original version, stored all recipient addresses as a
+		//    RFC822 string
+		// 2: Stores recipients in separate properties,
+		//    filters/indexes all attachments
+		private const int INDEX_VERSION = 2;
 
-		public EvolutionMailQueryable () : base ("MailIndex")
+		public EvolutionMailQueryable () : base ("MailIndex", INDEX_VERSION)
 		{
 			this.local_path = Path.Combine (PathFinder.HomeDir, ".evolution/mail/local");
 			this.imap_path = Path.Combine (PathFinder.HomeDir, ".evolution/mail/imap");
 			this.imap4_path = Path.Combine (PathFinder.HomeDir, ".evolution/mail/imap4");
 		}
 
-		private void Crawl ()
-		{
-			crawler.Crawl ();
-			foreach (FileInfo file in crawler.Summaries)
-				IndexSummary (file);
-			foreach (FileInfo file in crawler.Mboxes)
-				IndexMbox (file);
-		}
-
 		private void CrawlHook (Scheduler.Task task)
 		{
-			Crawl ();
+			crawler.Crawl ();
 			task.Reschedule = true;
 			task.TriggerTime = DateTime.Now.AddSeconds (polling_interval_in_seconds);
 		}
 
+		//////////////////////////////////////////////////////////////////////////////////////////////
+
 		private void StartWorker ()
 		{
 			Logger.Log.Info ("Starting Evolution mail backend");
@@ -84,28 +79,23 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			// Check that we have data to index
 			if ((! Directory.Exists (this.local_path)) && (! Directory.Exists (this.imap_path))) {
 				// No mails present, repoll every minute
-				log.Warn ("Evolution mail store not found, watching for it.");
+				Logger.Log.Warn ("Evolution mail store not found, watching for it.");
 				GLib.Timeout.Add (60000, new GLib.TimeoutHandler (CheckForMailData));
 				return;
 			}
 
-			// Get notification when an index or summary file changes
-			if (Inotify.Enabled) {
-				Inotify.Event += OnInotifyEvent;
-				Watch (this.local_path);
-				Watch (this.imap_path);
-				Watch (this.imap4_path);
-			}
-
 			Logger.Log.Debug ("Starting mail crawl");
 			crawler = new MailCrawler (this.local_path, this.imap_path, this.imap4_path);
-			Crawl ();
+			crawler.MboxAddedEvent += IndexMbox;
+			crawler.SummaryAddedEvent += IndexSummary;
+			crawler.Crawl ();
 			Logger.Log.Debug ("Mail crawl finished");
 
 			// If we don't have inotify, we have to poll the file system.  Ugh.
 			if (! Inotify.Enabled) {
 				Scheduler.Task task = Scheduler.TaskFromHook (new Scheduler.TaskHook (CrawlHook));
 				task.Tag = "Crawling ~/.evolution to find summary changes";
+				task.Source = this;
 				ThisScheduler.Add (task);
 			}
 
@@ -116,81 +106,11 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 
 		public override void Start () 
 		{
-			Logger.Log.Info ("Starting Evolution mail backend");
 			base.Start ();
 			
 			ExceptionHandlingThread.Start (new ThreadStart (StartWorker));
 		}
 
-		private void Watch (string path)
-		{
-			DirectoryInfo root = new DirectoryInfo (path);
-			if (! root.Exists)
-				return;
-
-			Queue queue = new Queue ();
-			queue.Enqueue (root);
-
-			while (queue.Count > 0) {
-				DirectoryInfo dir = queue.Dequeue () as DirectoryInfo;
-
-				if (! dir.Exists)
-					continue;
-				
-				int wd = Inotify.Watch (dir.FullName,
-							Inotify.EventType.CreateSubdir
-							| Inotify.EventType.DeleteSubdir
-							| Inotify.EventType.MovedTo);
-				watched [wd] = dir.FullName;
-
-				foreach (DirectoryInfo subdir in dir.GetDirectories ())
-					queue.Enqueue (subdir);
-			}
-		}
-
-		private void Ignore (string path)
-		{
-			Inotify.Ignore (path);
-			watched.RemoveAt (watched.IndexOfValue (path));
-		}
-
-		private void OnInotifyEvent (int wd,
-					     string path,
-					     string subitem,
-					     string srcpath,
-					     Inotify.EventType type)
-		{
-			if (subitem == "" || ! watched.Contains (wd))
-				return;
-
-			string fullPath = Path.Combine (path, subitem);
-
-			switch (type) {
-				
-			case Inotify.EventType.CreateSubdir:
-				Watch (fullPath);
-				break;
-
-			case Inotify.EventType.DeleteSubdir:
-				Ignore (fullPath);
-				break;
-
-			case Inotify.EventType.MovedTo:
-				if (subitem == "summary") {
-					// IMAP summary
-					log.Info ("Reindexing updated IMAP summary: {0}", fullPath);
-					this.IndexSummary (new FileInfo (fullPath));
-				} else if (Path.GetExtension (fullPath) == ".ev-summary") {
-					// mbox summary
-					string mbox_file = Path.ChangeExtension (fullPath, null);
-					log.Info ("Reindexing updated mbox: {0}", mbox_file);
-					this.IndexMbox (new FileInfo (mbox_file));
-				}
-
-				break;
-			}
-		}
-
 		private bool CheckForMailData ()
 		{
 			if ((! Directory.Exists (this.local_path)) && (! Directory.Exists (this.imap_path)))
@@ -207,18 +127,35 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 
 		public void IndexSummary (FileInfo summaryInfo)
 		{
+			// If there's already a task running for this folder,
+			// don't interrupt it.
+			if (ThisScheduler.ContainsByTag (summaryInfo.FullName)) {
+				Logger.Log.Debug ("Not adding task for already running task: {0}", summaryInfo.FullName);
+				return;
+			}
+
+			Logger.Log.Debug ("Will index summary {0}", summaryInfo.FullName);
 			EvolutionMailIndexableGeneratorImap generator = new EvolutionMailIndexableGeneratorImap (this, summaryInfo);
 			Scheduler.Task task;
-			task = NewAddTask (generator, new Scheduler.Hook (generator.Checkpoint));
+			task = NewAddTask (generator);
+			task.Tag = summaryInfo.FullName;
 			// IndexableGenerator tasks default to having priority Scheduler.Priority Generator
 			ThisScheduler.Add (task);
 		}
 
 		public void IndexMbox (FileInfo mboxInfo)
 		{
+			// If there's already a task running for this mbox,
+			// don't interrupt it.
+			if (ThisScheduler.ContainsByTag (mboxInfo.FullName)) {
+				Logger.Log.Debug ("Not adding task for already running task: {0}", mboxInfo.FullName);
+				return;
+			}
+
+			Logger.Log.Debug ("Will index mbox {0}", mboxInfo.FullName);
 			EvolutionMailIndexableGeneratorMbox generator = new EvolutionMailIndexableGeneratorMbox (this, mboxInfo);
 			Scheduler.Task task;
-			task = NewAddTask (generator, new Scheduler.Hook (generator.Checkpoint));
+			task = NewAddTask (generator);
 			task.Tag = mboxInfo.FullName;
 			// IndexableGenerator tasks default to having priority Scheduler.Priority Generator
 			ThisScheduler.Add (task);
@@ -232,8 +169,19 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 
 		override protected double RelevancyMultiplier (Hit hit)
 		{
-			return HalfLifeMultiplierFromProperty (hit, 0.25,
-							       "fixme:received", "fixme:sentdate");
+			double t = 1.0;
+
+			// FIXME: We should probably be more careful
+			// about how we combine these two numbers into one,
+			// since the cardinal value of the score is used for
+			// comparisons with hits of other types.  It isn't
+			// sufficient to just worry about the ordinal relationship
+			// between two scores.
+			t *= HalfLifeMultiplierFromProperty (hit, 0.25,
+							     "fixme:received", "fixme:sentdate");
+
+
+			return t;
 		}
 	}
 
