@@ -52,15 +52,6 @@ namespace Beagle.Daemon {
 
 		//////////////////////////
 
-		private bool delete_content;
-
-		public bool DeleteContent {
-			get { return this.delete_content; }
-			set { this.delete_content = value; }
-		}
-
-		//////////////////////////
-
 		private ArrayList supported_flavors = new ArrayList ();
 		
 		protected void AddSupportedFlavor (FilterFlavor flavor) 
@@ -197,11 +188,6 @@ namespace Beagle.Daemon {
 		private ArrayList hotPool;
 		private ArrayList propertyPool;
 		
-		private int max_text_pool_len = 0;
-		private int max_text_pool_size = 0;
-		private int max_hot_pool_len = 0;
-		
-
 		private bool last_was_structural_break = true;
 
 		// This two-arg AppendText() will give flexibility to
@@ -255,19 +241,6 @@ namespace Beagle.Daemon {
 			if (str != null) {
 				textPool.Add (str);
 
-				int pool_size = 0;
-				foreach (string x in textPool)
-					pool_size += x.Length;
-
-				if (pool_size > max_text_pool_size)
-					max_text_pool_size = pool_size;
-
-				if (textPool.Count > max_text_pool_len)
-					max_text_pool_len = textPool.Count;
-
-				if (hotPool.Count > max_hot_pool_len)
-					max_hot_pool_len = hotPool.Count;
-
 				if (snippetWriter != null)
 					snippetWriter.Write (str);
 
@@ -340,6 +313,7 @@ namespace Beagle.Daemon {
 
 		protected void Error ()
 		{
+			Cleanup (); // force the clean-up of temporary files on an error
 			has_error = true;
 		}
 
@@ -385,21 +359,69 @@ namespace Beagle.Daemon {
 		private FileStream currentStream = null;
 		private StreamReader currentReader = null;
 
+		public bool Open (TextReader reader)
+		{
+			tempFile = Path.GetTempFileName ();
+                        FileStream file_stream = File.OpenWrite (tempFile);
+
+			//Logger.Log.Debug ("Storing text in tempFile {0}", tempFile);
+
+                        // When we dump the contents of a reader into a file, we
+                        // expect to use it again soon.
+                        FileAdvise.PreLoad (file_stream);
+
+                        // Make sure the temporary file is only readable by the owner.
+                        // FIXME: There is probably a race here.  Could some malicious program
+                        // do something to the file between creation and the chmod?
+                        Mono.Posix.Syscall.chmod (tempFile, (Mono.Posix.FileMode) 256);
+
+                        BufferedStream buffered_stream = new BufferedStream (file_stream);
+                        StreamWriter writer = new StreamWriter (buffered_stream);
+
+                        const int BUFFER_SIZE = 8192;
+                        char [] buffer = new char [BUFFER_SIZE];
+
+                        int read;
+                        do {
+                                read = reader.Read (buffer, 0, BUFFER_SIZE);
+                                if (read > 0)
+                                        writer.Write (buffer, 0, read);
+                        } while (read > 0);
+
+                        writer.Close ();
+
+			return Open (new FileInfo (tempFile));
+		}
+		
 		public bool Open (Stream stream)
 		{
-			// If we are handed a stream, dump it into
-			// a temporary file.
-			tempFile = Path.GetTempFileName();
-			Stream tempStream = File.OpenWrite (tempFile);
+			tempFile = Path.GetTempFileName ();
+                        FileStream file_stream = File.OpenWrite (tempFile);
+			
+			//Logger.Log.Debug ("Storing stream in tempFile {0}", tempFile);
 
-			const int BUFFER_SIZE = 8192;
-			byte[] buffer = new byte [BUFFER_SIZE];
-			int n;
-			while ((n = stream.Read (buffer, 0, BUFFER_SIZE)) > 0) {
-				tempStream.Write (buffer, 0, n);
-			}
+                        // When we dump the contents of a reader into a file, we
+                        // expect to use it again soon.
+                        FileAdvise.PreLoad (file_stream);
 
-			tempStream.Close ();
+                        // Make sure the temporary file is only readable by the owner.
+                        // FIXME: There is probably a race here.  Could some malicious program
+                        // do something to the file between creation and the chmod?
+                        Mono.Posix.Syscall.chmod (tempFile, (Mono.Posix.FileMode) 256);
+
+                        BufferedStream buffered_stream = new BufferedStream (file_stream);
+
+                        const int BUFFER_SIZE = 8192;
+                        byte [] buffer = new byte [BUFFER_SIZE];
+
+                        int read;
+                        do {
+                                read = stream.Read (buffer, 0, BUFFER_SIZE);
+                                if (read > 0)
+                                        buffered_stream.Write (buffer, 0, read);
+                        } while (read > 0);
+
+                        buffered_stream.Close ();
 
 			return Open (new FileInfo (tempFile));
 		}
@@ -459,6 +481,7 @@ namespace Beagle.Daemon {
 					return false;				
 			} catch (Exception e) {
 				Logger.Log.Warn ("Unable to filter {0}: {1}", info.FullName, e.Message);
+				Cleanup (); // clean up temporary files on an exception
 				return false;
 			}
 
@@ -512,6 +535,8 @@ namespace Beagle.Daemon {
 
 		private void Close ()
 		{
+			Cleanup ();
+
 			if (currentStream == null)
 				return;
 
@@ -530,53 +555,55 @@ namespace Beagle.Daemon {
 
 			if (snippetWriter != null)
 				snippetWriter.Close ();
+		}
 
-			if (tempFile != null)
-				File.Delete (tempFile);
-
-			if (currentInfo != null && this.DeleteContent) {
+		public void Cleanup ()
+		{
+			if (tempFile != null) {
 				try {
-					currentInfo.Delete ();
-				} catch (Exception e) {
-					Logger.Log.Debug ("Caught exception trying to delete {0} in filter '{1}'",
-							  currentInfo.FullName, Name);
-					Logger.Log.Debug (e);					
+					File.Delete (tempFile);
+				} catch (Exception ex) {
+					// Just in case it is gone already
 				}
+				tempFile = null;
 			}
 		}
 
-		private string PullFromArray (ArrayList array)
+		private bool PullFromArray (ArrayList array, StringBuilder sb)
 		{
 			while (array.Count == 0 && Pull ()) { }
+
 			if (array.Count > 0) {
-				string str = (string) array [0];
-				array.RemoveAt (0);
-				return str;
+				foreach (string str in array)
+					sb.Append (str);
+
+				array.Clear ();
+				return true;
 			}
-			return null;
+			return false;
 		}
 
-		private string PullTextCarefully (ArrayList array)
+		private bool PullTextCarefully (ArrayList array, StringBuilder sb)
 		{
-			string text = null;
-			try {	
-				text = PullFromArray (array);
+			bool pulled = false;
+			try {
+				pulled = PullFromArray (array, sb);
 			} catch (Exception ex) {
 				Logger.Log.Debug ("Caught exception while pulling text in filter '{0}'", Name);
 				Logger.Log.Debug (ex);
 			}
-			return text;
 
+			return pulled;
 		}
 
-		private string PullText ()
+		private bool PullText (StringBuilder sb)
 		{
-			return PullTextCarefully (textPool);
+			return PullTextCarefully (textPool, sb);
 		}
 
-		private string PullHotText ()
+		private bool PullHotText (StringBuilder sb)
 		{
-			return PullTextCarefully (hotPool);
+			return PullTextCarefully (hotPool, sb);
 		}
 
 		public TextReader GetTextReader ()
