@@ -90,6 +90,36 @@ namespace Beagle.Daemon {
 
 		////////////////////////////////////////////////////////////////
 
+		public Uri[] PropertyQuery (Property prop)
+		{
+			// FIXME: Should we support scanning the secondary
+			// index as well?
+
+			IndexReader primary_reader;
+			LNS.IndexSearcher primary_searcher;
+
+			primary_reader = IndexReader.Open (PrimaryStore);
+			primary_searcher = new LNS.IndexSearcher (primary_reader);
+
+			Term term = new Term (PropertyToFieldName (prop.Type, prop.Key), prop.Value);
+			LNS.TermQuery query = new LNS.TermQuery (term);
+			LNS.Hits hits = primary_searcher.Search (query);
+
+			Uri[] uri_list = new Uri [hits.Length ()];
+			for (int i = 0; i < hits.Length (); i++) {
+				Document doc;
+				doc = hits.Doc (i);
+				uri_list [i] = GetUriFromDocument (doc);
+			}
+
+			primary_reader.Close ();
+			primary_searcher.Close ();
+
+			return uri_list;
+		}
+
+		////////////////////////////////////////////////////////////////
+
 		// Returns the lowest matching score before the results are
 		// truncated.
 		public void DoQuery (Query               query,
@@ -273,21 +303,21 @@ namespace Beagle.Daemon {
 			sw.Reset ();
 			sw.Start ();
 
-			// If we didn't get any matches, give up.
-			if (primary_matches == null || ! primary_matches.ContainsTrue ())
-				return;
+			// Only generate results if we got some matches
+			if (primary_matches != null && primary_matches.ContainsTrue ()) {
+				GenerateQueryResults (primary_reader,
+						      primary_searcher,
+						      secondary_searcher,
+					    	  primary_matches,
+						      result,
+						      term_list,
+						      query.MaxHits,
+						      DateTime.MinValue,
+						      DateTime.MaxValue,
+						      uri_filter,
+					    	  hit_filter);
+			}
 
-			GenerateQueryResults (primary_reader,
-					      primary_searcher,
-					      secondary_searcher,
-					      primary_matches,
-					      result,
-					      term_list,
-					      query.MaxHits,
-					      DateTime.MinValue,
-					      DateTime.MaxValue,
-					      uri_filter,
-					      hit_filter);
 			//
 			// Finally, we clean up after ourselves.
 			//
