@@ -57,35 +57,36 @@ namespace Beagle.Daemon {
 					++pos;
 					continue;
 				}
-
+				
 				// Find the end of the next token
 				int next_pos = pos+1;
 				while (next_pos < text.Length && !IsTokenSeparator (text [next_pos]))
 					++next_pos;
-
+				
 				string stemmed_token = null;
-
+				int hl_offset = 0;
+				
 				// Iterate through the stemmed terms and match the token
 				for (int i = 0; i < stemmed_terms.Length; i++) {
-
+					
 					// If this term is longer than the token in question, give up.
 					if (next_pos - pos < stemmed_terms [i].Length)
 						continue;
-
+					
 					// Make sure this isn't a stop word.
-					if (LuceneDriver.IsStopWord (stemmed_terms [i]))
+					if (LuceneCommon.IsStopWord (stemmed_terms [i]))
 						continue;
 
 					// We cache the token, so as to avoid stemming it more than once
 					// when considering multiple terms.
 					if (stemmed_token == null) {
 						string token = text.Substring (pos, next_pos - pos).ToLower ();
-						stemmed_token = LuceneDriver.Stem (token);
+						stemmed_token = LuceneCommon.Stem (token);
 					}
 
 					if (stemmed_terms [i] != stemmed_token)
 						continue;
-
+					
 					// We have a match!
 
 				        int start_pos = pos;
@@ -96,6 +97,7 @@ namespace Beagle.Daemon {
 						if ((text[start_pos] == ' '))
 							count++;
 					}
+
 					if (start_pos != 0)
 						start_pos += 2;
 
@@ -103,6 +105,7 @@ namespace Beagle.Daemon {
 						if (text[stop_pos] == ' ')
 							count++;
 					}
+
 					if (stop_pos != text.Length)
 						stop_pos--;
 
@@ -116,7 +119,7 @@ namespace Beagle.Daemon {
 
 					string new_match = String.Concat (text.Substring (start_pos, pos - start_pos),
 									  "<font color=\"",
-									  colors [i%colors.Length],
+									  colors [(i - hl_offset) % colors.Length],
 									  "\"><b>",
 									  text.Substring (pos, next_pos-pos),
 									  "</b></font>",
@@ -132,7 +135,7 @@ namespace Beagle.Daemon {
 
 					prev_stop_pos = stop_pos;
 					prev_end_pos = next_pos;
-					
+
 					break;
 				}
 
@@ -142,11 +145,9 @@ namespace Beagle.Daemon {
 			// Add trailing match
 			if (prev_match != "")
 				matches.Add (prev_match); 
-
-			return;
 		}
 
-		static string[] colors = new string [] { "red", "blue", "green", "orange", "purple", "brown"};
+		static string[] colors = new string [] {"red", "blue", "green", "orange", "purple", "brown"};
 
 		const int soft_snippet_limit = 400;
 
@@ -159,20 +160,22 @@ namespace Beagle.Daemon {
 			if (string_source == null)
 				return null;
 
+#if false
 			int N = query_terms.Length;
 			string[] stemmed_terms = new string [N];
 			for (int i = 0; i < N; ++i) {
 				string term = query_terms [i];
 				if (term [0] == '-')
 					continue;
-				stemmed_terms [i] = LuceneDriver.Stem (query_terms [i]).ToLower ();
+				stemmed_terms [i] = LuceneCommon.Stem (query_terms [i]).ToLower ();
 			}
+#endif
 			
 			ArrayList matches = new ArrayList ();
 
 			string str;
 			while ( (str = string_source ()) != null) {
-				HighlightTerms (stemmed_terms, str, ref matches);
+				HighlightTerms (query_terms, str, ref matches);
 			}
 
 			string snippet = "";
